package orderprocessing;
/*
	History
	date		description
	-----		-----------
	11/12/99	Change date to CalculationCalendar,
	            format output
*/

import java.util.ArrayList;
import java.util.Iterator;
import java.io.Serializable;
import java.text.NumberFormat;
import java.util.HashMap;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.BufferedWriter;

/** This class represents what will ship from a customer order.
*	<p>
*	Additionally, this class maintains a collection of all Invoices.
*/

public class Invoice implements Serializable
{	/** Create an Invoice for a particular customer and a sequence of items ordered
	* @param order The order from which this invoice originates
	* @param shipableItems A ArrayList of OrderItems representing the items shipped
	*/
	public Invoice(Order order, ArrayList shipableItems)
	{	this.shipableItems = (ArrayList) shipableItems.clone();
		shipDate = order.requestedShippingDate();
		this.order = order;
		key = IDGenerator.nextID();
		allInvoices.put(key, this);
        invoiceDate = shipDate();
	}

	// accessors
	public Customer customer(){return order.customer();}
	public Iterator shipableItems(){return shipableItems.iterator();}
    public int size()
    {	return shipableItems.size();
    }
	public CalculationCalendar shipDate(){return shipDate;}
    public CalculationCalendar invoiceDate()
    {	return invoiceDate;
    }

    public Order order()
    {	return order;
    }

	/** Return the invoice number
    *	@return the invoice number
	*/
	public String key(){return key;}

	public CalculationCalendar paymentDueDate()
	{	return order.billingTerms().dueDate(invoiceDate); // is this OK?
	}

	/** Display the information about shipableItems will ship, including extended prices and a total
	*/
	public void display()
	{	System.out.println("\t\t\t------- Invoice -------");
	    System.out.println("Invoice No: " + key);
	    //System.out.println("Customer : " );
	    order.customer().display();
        System.out.println("Invoice Date: " + invoiceDate.getTime());
		Iterator items = shipableItems();
		double total = 0.0;

	    System.out.println("\n\n\tQty\tItem No. " +
	            "\tDescription\t\tsub-total\n");

	    while(items.hasNext())
		{	OrderItem ordered = (OrderItem)items.next();
			ordered.displayExtended();
			total += ordered.quantity() * ordered.unitPrice();
		}

		System.out.println("\n\t\t\t\tInvoice Total: "
		    + NumberFormat.getInstance().format(total));
	    System.out.println("-----------------------------------------------------------------");
	}

	public boolean equals(Object other)
	{	if(!(other instanceof Invoice))return false;
		return key.equals(((Invoice)other).key);
	}

	public int hashCode()
	{	return key.hashCode()*997;
	}

	public String toString(){return "Invoice with key: " + key;}

	public static void formatAll(BufferedWriter file) // NOTE: NOT in invoice number order.
	{   Iterator invoices = invoices();
	    Invoice invoice = null;
	    while (invoices.hasNext())
	    {	try
	    	{	invoice = (Invoice)invoices.next();
            	invoice.format(file);
				file.newLine();
	        	file.newLine();
	    	}
	        catch(IOException ex)
	        {	System.out.println("Failed to write invoice "+invoice.key() +"  " + ex);
	        }
	    }
	}

    // To do this in invoice number order you can iterate over the keys to extract them
    // into an ArrayList. Sort that list and then use the sorted list to get the keys
    // in sorted order to get the records themselves.

	public void format(BufferedWriter file) throws IOException
	{	// to do
	}

	private ArrayList shipableItems = null; // ArrayList of OrderItems
	private String key;
        private CalculationCalendar invoiceDate = null;
	private CalculationCalendar shipDate;
	private Order order;

	private static class IDGenerator
	{	public static String nextID()
	    {	return ID_PREFIX + Long.toString(nextID++);
	    }

	    private static long             nextID = 6001;
	    private final static String     ID_PREFIX = "I";
	}

    // Following is the collection of all invoices.
	private static HashMap allInvoices = new HashMap();

    public static Invoice get(String key)
    {	return (Invoice)allInvoices.get(key);
    }

	public static final void close(ObjectOutputStream out) throws IOException
	{	out.writeLong(IDGenerator.nextID);
		out.writeObject(allInvoices);
	}

	public static final void open(ObjectInputStream in) throws IOException, ClassNotFoundException
	{	IDGenerator.nextID = in.readLong();
		allInvoices = (HashMap)in.readObject();
	}

	public static Iterator invoices()
	{	return allInvoices.values().iterator();
	}

	public static Iterator keys()
	{	return allInvoices.keySet().iterator();
	}

	public static void dump()
	{	System.out.println("\n\n\n Invoices \n\n\n");
		System.out.println(allInvoices);
	}
}