package orderprocessing;

import junit.framework.*;
import java.util.ArrayList;
import java.util.Iterator;

public class InvoiceTest extends TestCase
{	private Invoice i1;
	private ArrayList items;
	private Order o1;

	public InvoiceTest(String s){super(s);} // Always required. Omitted from documentation.

	public void setUp()
	{	OrderProcessor.open(OrderProcessor.databaseName());//don't close.
    	items = new ArrayList();
		items.add("15~11232~16.95~Widget");
		o1 = OrderProcessor.orderProcessor.newOrder("C5002","NET 30","default", items ,new CalculationCalendar(),new CalculationCalendar(),new CalculationCalendar());
		i1 = OrderProcessor.orderProcessor.generateInvoice(o1);
	}

	protected void tearDown() throws Exception
	{	super.tearDown();
	}

	public void testItems()
	{	Iterator e = i1.shipableItems();
		assert("Empty invoice failure", e.hasNext());
		assert("Non-null invoice item failure", e.next() != null); // non empty and no null elements.
	}

    public void testInfo()
    {	assertEquals("Customer failure", i1.customer(), Customer.get("C5002"));
		assertEquals("BillingTerms match failure", o1.billingTerms(), BillingTerms.get("NET 30"));
        assertEquals("Ship address failure", o1.shippingAddress(), o1.customer().shippingAddress());

        // write some assertions on dates....

        CalculationCalendar today = new CalculationCalendar();
        CalculationCalendar todayPlus7 = today.incrementedBy(CalculationCalendar.DATE, 7);
        CalculationCalendar thenPlus10 = todayPlus7.incrementedBy(CalculationCalendar.DATE,10);
        assert("Requested ship date too soon failure", ! o1.requestedShippingDate().before(todayPlus7));
        assert("Cancel date too soon failure.", ! o1.cancelDate().before(thenPlus10));



    }

	public static Test suite()
	{	TestSuite suite = new TestSuite();
		suite.addTest(new InvoiceTest("testItems"));
        suite.addTest(new InvoiceTest("testInfo"));
		return suite;
	}

	public static void main(String [] args) // The batch interface:
	{	junit.textui.TestRunner.run(suite());
	}

}